@@ -179,7 +179,7 @@ class ScriptTranslator(object):
             tl = self.language_translates.get((identifier, language), None)
 
             if (tl is None) and alternate:
-                tl = self.language_translates.get((identifier, language), alternate)
+                tl = self.language_translates.get((identifier, language), None)
 
         else:
             tl = None
@@ -444,8 +444,12 @@ Default = renpy.object.Sentinel("default")
 
 def translate_string(s, language=Default):
     """
-    Translates interface string `s` to `language`. If `languages` is Default,
-    uses the language set in the preferences.
+    :doc: translate_string
+    :name: renpy.translate_string
+
+    Translates interface string `s` to `language`. If `language` is Default,
+    uses the language set in the preferences. This does not mark `s` to be
+    translated.
     """
 
     if language is Default:
@@ -573,8 +577,6 @@ def old_change_language(tl, language):
 
 def new_change_language(tl, language):
 
-    renpy.config.init_system_styles()
-
     for i in tl.python[language]:
         renpy.python.py_exec_bytecode(i.code.bytecode)
 
@@ -596,6 +598,8 @@ def new_change_language(tl, language):
 
     renpy.game.invoke_in_new_context(run_blocks)
 
+    renpy.config.init_system_styles()
+
 
 def change_language(language, force=False):
     """
@@ -677,3 +681,220 @@ def known_languages():
     """
 
     return { i for i in renpy.game.script.translator.languages if i is not None }  # @UndefinedVariable
+
+################################################################################
+# Detect language
+################################################################################
+
+locales = {
+    "ab": "abkhazian",
+    "aa": "afar",
+    "af": "afrikaans",
+    "ak": "akan",
+    "sq": "albanian",
+    "am": "amharic",
+    "ar": "arabic",
+    "an": "aragonese",
+    "hy": "armenian",
+    "as": "assamese",
+    "av": "avaric",
+    "ae": "avestan",
+    "ay": "aymara",
+    "az": "azerbaijani",
+    "bm": "bambara",
+    "ba": "bashkir",
+    "eu": "basque",
+    "be": "belarusian",
+    "bn": "bengali",
+    "bh": "bihari",
+    "bi": "bislama",
+    "bs": "bosnian",
+    "br": "breton",
+    "bg": "bulgarian",
+    "my": "burmese",
+    "ca": "catalan",
+    "ch": "chamorro",
+    "ce": "chechen",
+    "ny": "chewa",
+    "cv": "chuvash",
+    "kw": "cornish",
+    "co": "corsican",
+    "cr": "cree",
+    "hr": "croatian",
+    "cs": "czech",
+    "da": "danish",
+    "dv": "maldivian",
+    "nl": "dutch",
+    "dz": "dzongkha",
+    "en": "english",
+    "et": "estonian",
+    "ee": "ewe",
+    "fo": "faroese",
+    "fj": "fijian",
+    "fi": "finnish",
+    "fr": "french",
+    "ff": "fulah",
+    "gl": "galician",
+    "ka": "georgian",
+    "de": "german",
+    "el": "greek",
+    "gn": "guaran",
+    "gu": "gujarati",
+    "ht": "haitian",
+    "ha": "hausa",
+    "he": "hebrew",
+    "hz": "herero",
+    "hi": "hindi",
+    "ho": "hiri_motu",
+    "hu": "hungarian",
+    "id": "indonesian",
+    "ga": "irish",
+    "ig": "igbo",
+    "ik": "inupiaq",
+    "is": "icelandic",
+    "it": "italian",
+    "iu": "inuktitut",
+    "ja": "japanese",
+    "jv": "javanese",
+    "kl": "greenlandic",
+    "kn": "kannada",
+    "kr": "kanuri",
+    "ks": "kashmiri",
+    "kk": "kazakh",
+    "km": "khmer",
+    "ki": "kikuyu",
+    "rw": "kinyarwanda",
+    "ky": "kirghiz",
+    "kv": "komi",
+    "kg": "kongo",
+    "ko": "korean",
+    "ku": "kurdish",
+    "kj": "kuanyama",
+    "la": "latin",
+    "lb": "luxembourgish",
+    "lg": "ganda",
+    "li": "limburgan",
+    "ln": "lingala",
+    "lo": "lao",
+    "lt": "lithuanian",
+    "lv": "latvian",
+    "gv": "manx",
+    "mk": "macedonian",
+    "mg": "malagasy",
+    "ms": "malay",
+    "ml": "malayalam",
+    "mt": "maltese",
+    "mi": "maori",
+    "mr": "marathi",
+    "mh": "marshallese",
+    "mn": "mongolian",
+    "na": "nauru",
+    "nv": "navaho",
+    "ne": "nepali",
+    "ng": "ndonga",
+    "no": "norwegian",
+    "ii": "nuosu",
+    "nr": "ndebele",
+    "oc": "occitan",
+    "oj": "ojibwa",
+    "om": "oromo",
+    "or": "oriya",
+    "os": "ossetian",
+    "pa": "panjabi",
+    "pi": "pali",
+    "fa": "persian",
+    "pl": "polish",
+    "ps": "pashto",
+    "pt": "portuguese",
+    "qu": "quechua",
+    "rm": "romansh",
+    "rn": "rundi",
+    "ro": "romanian",
+    "ru": "russian",
+    "sa": "sanskrit",
+    "sc": "sardinian",
+    "sd": "sindhi",
+    "se": "sami",
+    "sm": "samoan",
+    "sg": "sango",
+    "sr": "serbian",
+    "gd": "gaelic",
+    "sn": "shona",
+    "si": "sinhala",
+    "sk": "slovak",
+    "sl": "slovene",
+    "so": "somali",
+    "st": "sotho",
+    "es": "spanish",
+    "su": "sundanese",
+    "sw": "swahili",
+    "ss": "swati",
+    "sv": "swedish",
+    "ta": "tamil",
+    "te": "telugu",
+    "tg": "tajik",
+    "th": "thai",
+    "ti": "tigrinya",
+    "bo": "tibetan",
+    "tk": "turkmen",
+    "tl": "tagalog",
+    "tn": "tswana",
+    "to": "tongan",
+    "tr": "turkish",
+    "ts": "tsonga",
+    "tt": "tatar",
+    "tw": "twi",
+    "ty": "tahitian",
+    "ug": "uighur",
+    "uk": "ukrainian",
+    "ur": "urdu",
+    "uz": "uzbek",
+    "ve": "venda",
+    "vi": "vietnamese",
+    "wa": "walloon",
+    "cy": "welsh",
+    "wo": "wolof",
+    "fy": "frisian",
+    "xh": "xhosa",
+    "yi": "yiddish",
+    "yo": "yoruba",
+    "za": "zhuang",
+    "zu": "zulu",
+    "chs": "simplified_chinese",
+    "cht": "traditional_chinese",
+    "zh": "traditional_chinese",
+}
+
+def detect_user_locale():
+    import locale
+    if renpy.windows:
+        import ctypes
+        windll = ctypes.windll.kernel32
+        locale_name = locale.windows_locale.get(windll.GetUserDefaultUILanguage())
+    elif renpy.android:
+        from jnius import autoclass
+        Locale = autoclass('java.util.Locale')
+        locale_name = str(Locale.getDefault().getLanguage())
+    elif renpy.ios:
+        import pyobjus
+        NSLocale = pyobjus.autoclass("NSLocale")
+        languages = NSLocale.preferredLanguages()
+        locale_name = languages.objectAtIndex_(0).UTF8String().decode("utf-8")
+        locale_name.replace("-", "_")
+    else:
+        locale_name = locale.getdefaultlocale()
+        if locale_name is not None:
+            locale_name = locale_name[0]
+
+    if locale_name is None:
+        return None, None
+
+    normalize = locale.normalize(locale_name)
+    if normalize == locale_name:
+        language = region = locale_name
+    else:
+        locale_name = normalize
+        if '.' in locale_name:
+            locale_name, _ = locale_name.split('.', 1)
+        language, region = locale_name.lower().split("_")
+    return language, region
